@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -20,12 +20,16 @@
 #include "xdebug_private.h"
 #include "xdebug_set.h"
 #include "xdebug_var.h"
+#include "xdebug_branch_info.h"
 #include "xdebug_code_coverage.h"
 #include "xdebug_compat.h"
 #include "xdebug_tracing.h"
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
+static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa TSRMLS_DC);
+static char* xdebug_func_format(xdebug_func *func TSRMLS_DC);
+
 void xdebug_coverage_line_dtor(void *data)
 {
 	xdebug_coverage_line *line = (xdebug_coverage_line *) data;
@@ -33,15 +37,83 @@ void xdebug_coverage_line_dtor(void *data)
 	xdfree(line);
 }
 
+xdebug_coverage_file *xdebug_coverage_file_ctor(char *filename)
+{
+	xdebug_coverage_file *file;
+
+	file = xdmalloc(sizeof(xdebug_coverage_file));
+	file->name = xdstrdup(filename);
+	file->lines = xdebug_hash_alloc(128, xdebug_coverage_line_dtor);
+	file->functions = xdebug_hash_alloc(128, xdebug_coverage_function_dtor);
+	file->has_branch_info = 0;
+
+	return file;
+}
+
 void xdebug_coverage_file_dtor(void *data)
 {
 	xdebug_coverage_file *file = (xdebug_coverage_file *) data;
 
 	xdebug_hash_destroy(file->lines);
+	xdebug_hash_destroy(file->functions);
 	xdfree(file->name);
 	xdfree(file);
 }
 
+xdebug_coverage_function *xdebug_coverage_function_ctor(char *function_name)
+{
+	xdebug_coverage_function *function;
+
+	function = xdmalloc(sizeof(xdebug_coverage_function));
+	function->name = xdstrdup(function_name);
+	function->branch_info = NULL;
+
+	return function;
+}
+
+void xdebug_coverage_function_dtor(void *data)
+{
+	xdebug_coverage_function *function = (xdebug_coverage_function *) data;
+
+	if (function->branch_info) {
+		xdebug_branch_info_free(function->branch_info);
+	}
+	xdfree(function->name);
+	xdfree(function);
+}
+
+void xdebug_print_opcode_info(char type, zend_execute_data *execute_data, zend_op *cur_opcode TSRMLS_DC)
+{
+	zend_op_array *op_array = execute_data->op_array;
+	char *file = (char *) op_array->filename;
+	xdebug_func func_info;
+	char *function_name;
+	long opnr = execute_data->opline - execute_data->op_array->opcodes;
+
+	xdebug_build_fname_from_oparray(&func_info, op_array TSRMLS_CC);
+	function_name = xdebug_func_format(&func_info TSRMLS_CC);
+	if (func_info.class) {
+		xdfree(func_info.class);
+	}
+	if (func_info.function) {
+		xdfree(func_info.function);
+	}
+
+	xdebug_branch_info_mark_reached(file, function_name, op_array, opnr TSRMLS_CC);
+	xdfree(function_name);
+}
+
+int xdebug_check_branch_entry_handler(ZEND_OPCODE_HANDLER_ARGS)
+{
+	if (XG(do_code_coverage)) {
+		zend_op *cur_opcode;
+		cur_opcode = *EG(opline_ptr);
+
+		xdebug_print_opcode_info('G', execute_data, cur_opcode TSRMLS_CC);
+	}
+	return ZEND_USER_OPCODE_DISPATCH;
+}
+
 #define XDEBUG_OPCODE_OVERRIDE(f) \
 	int xdebug_##f##_handler(ZEND_OPCODE_HANDLER_ARGS) \
 	{ \
@@ -63,6 +135,7 @@ int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS)
 
 		file = (char *)op_array->filename;
 
+		xdebug_print_opcode_info('C', execute_data, cur_opcode TSRMLS_CC);
 		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
 	}
 	return ZEND_USER_OPCODE_DISPATCH;
@@ -83,54 +156,50 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	next_opcode = cur_opcode + 1;
 	prev_opcode = cur_opcode - 1;
 
-	if (cur_opcode->XDEBUG_TYPE(op1) == IS_VAR &&
-			(next_opcode->XDEBUG_TYPE(op1) == IS_VAR || cur_opcode->XDEBUG_TYPE(op2) == IS_VAR) &&
+	if (cur_opcode->op1_type == IS_VAR &&
+			(next_opcode->op1_type == IS_VAR || cur_opcode->op2_type == IS_VAR) &&
 			prev_opcode->opcode == ZEND_FETCH_RW &&
-			prev_opcode->XDEBUG_TYPE(op1) == IS_CONST &&
-#if PHP_VERSION_ID >= 50399
+			prev_opcode->op1_type == IS_CONST &&
 			Z_TYPE_P(prev_opcode->op1.zv) == IS_STRING
-#else
-			prev_opcode->op1.u.constant.type == IS_STRING
-#endif
 	) {
-#if PHP_VERSION_ID >= 50399
 		xdebug_str_add(&name, xdebug_sprintf("$%s", Z_STRVAL_P(prev_opcode->op1.zv)), 1);
-#else
-		xdebug_str_add(&name, xdebug_sprintf("$%s", prev_opcode->op1.u.constant.value.str.val), 1);
-#endif
 	}
 
-	is_static = (prev_opcode->XDEBUG_TYPE(op1) == IS_CONST && prev_opcode->XDEBUG_EXTENDED_VALUE(op2) == ZEND_FETCH_STATIC_MEMBER);
+	is_static = (prev_opcode->op1_type == IS_CONST && prev_opcode->extended_value == ZEND_FETCH_STATIC_MEMBER);
 	options = xdebug_var_export_options_from_ini(TSRMLS_C);
 	options->no_decoration = 1;
 
-	if (cur_opcode->XDEBUG_TYPE(op1) == IS_CV) {
-		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->XDEBUG_ZNODE_ELEM(op1, var), &cv_len)), 1);
-	} else if (cur_opcode->XDEBUG_TYPE(op1) == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
+	if (cur_opcode->op1_type == IS_CV) {
+		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op1.var, &cv_len)), 1);
+	} else if (cur_opcode->op1_type == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
 		if (is_static) {
 			xdebug_str_add(&name, xdebug_sprintf("self::"), 1);
 		} else {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, prev_opcode->XDEBUG_TYPE(op1), &prev_opcode->op1, &is_var), 0, options);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, prev_opcode->op1_type, &prev_opcode->op1, &is_var), 0, options);
 			xdebug_str_add(&name, xdebug_sprintf("$%s", zval_value), 1);
 		}
 	} else if (is_static) { /* todo : see if you can change this and the previous cases around */
 		xdebug_str_add(&name, xdebug_sprintf("self::"), 1 );
 	}
-	if (cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR ) {
+	if (cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR
+#if PHP_VERSION_ID >= 50600
+		|| cur_opcode->opcode == ZEND_ASSIGN_POW
+#endif
+	) {
 		if (cur_opcode->extended_value == ZEND_ASSIGN_OBJ) {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var), 0, options);
-			if (cur_opcode->XDEBUG_TYPE(op1) == IS_UNUSED) {
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var), 0, options);
+			if (cur_opcode->op1_type == IS_UNUSED) {
 				xdebug_str_add(&name, xdebug_sprintf("$this->%s", zval_value), 1);
 			} else {
 				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
 			}
 		} else if (cur_opcode->extended_value == ZEND_ASSIGN_DIM) {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var), 0, NULL);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var), 0, NULL);
 			xdebug_str_add(&name,xdebug_sprintf("[%s]", zval_value), 1);
 		}
 	}
 	if (cur_opcode->opcode >= ZEND_PRE_INC_OBJ && cur_opcode->opcode <= ZEND_POST_DEC_OBJ) {
-		zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var), 0, options);
+		zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var), 0, options);
 		xdebug_str_add(&name, xdebug_sprintf("$this->%s", zval_value), 1);
 	}
 	if (zval_value) {
@@ -150,29 +219,29 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	if (gohungfound) {
 		do
 		{
-			if (opcode_ptr->XDEBUG_TYPE(op1) == IS_UNUSED && opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
+			if (opcode_ptr->op1_type == IS_UNUSED && opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
 				xdebug_str_add(&name, "$this", 0);
 			}
-			if (opcode_ptr->XDEBUG_TYPE(op1) == IS_CV) {
-				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->XDEBUG_ZNODE_ELEM(op1, var), &cv_len)), 1);
+			if (opcode_ptr->op1_type == IS_CV) {
+				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->op1.var, &cv_len)), 1);
 			}
 			if (opcode_ptr->opcode == ZEND_FETCH_W) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op1), &opcode_ptr->op1, &is_var), 0, options);
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op1_type, &opcode_ptr->op1, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
 			}
 			if (is_static && opcode_ptr->opcode == ZEND_FETCH_RW) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op1), &opcode_ptr->op1, &is_var), 0, options);
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op1_type, &opcode_ptr->op1, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
 			}
 			if (opcode_ptr->opcode == ZEND_FETCH_DIM_W) {
-				if (opcode_ptr->XDEBUG_TYPE(op2) != IS_VAR) {
-					zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, &is_var), 0, NULL);
+				if (opcode_ptr->op2_type != IS_VAR) {
+					zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op2_type, &opcode_ptr->op2, &is_var), 0, NULL);
 					xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
 				} else {
 					xdebug_str_add(&name, xdebug_sprintf("[???]") , 1);
 				}
 			} else if (opcode_ptr->opcode == ZEND_FETCH_OBJ_W) {
-				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, &is_var), 0, options);
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op2_type, &opcode_ptr->op2, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("->%s", zval_value), 1);
 			}
 			opcode_ptr = opcode_ptr + 1;
@@ -184,18 +253,18 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	}
 
 	if (cur_opcode->opcode == ZEND_ASSIGN_OBJ) {
-		if (cur_opcode->XDEBUG_TYPE(op1) == IS_UNUSED) {
+		if (cur_opcode->op1_type == IS_UNUSED) {
 			xdebug_str_add(&name, "$this", 0);
 		}
-		dimval = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var);
+		dimval = xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var);
 		xdebug_str_add(&name, xdebug_sprintf("->%s", Z_STRVAL_P(dimval)), 1);
 	}
 
 	if (cur_opcode->opcode == ZEND_ASSIGN_DIM) {
-		if (next_opcode->opcode == ZEND_OP_DATA && cur_opcode->XDEBUG_TYPE(op2) == IS_UNUSED) {
+		if (next_opcode->opcode == ZEND_OP_DATA && cur_opcode->op2_type == IS_UNUSED) {
 			xdebug_str_add(&name, "[]", 0);
 		} else {
-			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->XDEBUG_TYPE(op2), &opcode_ptr->op2, &is_var), 0, NULL);
+			zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op2_type, &opcode_ptr->op2, &is_var), 0, NULL);
 			xdebug_str_add(&name, xdebug_sprintf("[%s]", zval_value), 1);
 			xdfree(zval_value);
 		}
@@ -215,7 +284,6 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 	zend_op       *cur_opcode, *next_opcode;
 	char          *full_varname;
 	zval          *val = NULL;
-	char          *t;
 	int            is_var;
 	function_stack_entry *fse;
 
@@ -224,10 +292,14 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 	file = (char *) op_array->filename;
 	lineno = cur_opcode->lineno;
 
-	if (do_cc && XG(do_code_coverage)) {
-		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
+	if (XG(do_code_coverage)) {
+		xdebug_print_opcode_info('=', execute_data, cur_opcode TSRMLS_CC);
+	
+		if (do_cc) {
+			xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
+		}
 	}
-	if (XG(do_trace) && XG(trace_file) && XG(collect_assignments)) {
+	if (XG(do_trace) && XG(trace_context) && XG(collect_assignments)) {
 		full_varname = xdebug_find_var_name(execute_data TSRMLS_CC);
 
 		if (cur_opcode->opcode >= ZEND_PRE_INC && cur_opcode->opcode <= ZEND_POST_DEC) {
@@ -242,7 +314,7 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 			xdfree(full_varname);
 			full_varname = tmp_varname;
 
-			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op1), &cur_opcode->op1, &is_var);
+			val = xdebug_get_zval(execute_data, cur_opcode->op1_type, &cur_opcode->op1, &is_var);
 		} else if (cur_opcode->opcode >= ZEND_PRE_INC_OBJ && cur_opcode->opcode <= ZEND_POST_DEC_OBJ) {
 			char *tmp_varname;
 
@@ -255,19 +327,18 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 			xdfree(full_varname);
 			full_varname = tmp_varname;
 
-			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var);
+			val = xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var);
 		} else if (next_opcode->opcode == ZEND_OP_DATA) {
-			val = xdebug_get_zval(execute_data, next_opcode->XDEBUG_TYPE(op1), &next_opcode->op1, &is_var);
+			val = xdebug_get_zval(execute_data, next_opcode->op1_type, &next_opcode->op1, &is_var);
 		} else {
-			val = xdebug_get_zval(execute_data, cur_opcode->XDEBUG_TYPE(op2), &cur_opcode->op2, &is_var);
+			val = xdebug_get_zval(execute_data, cur_opcode->op2_type, &cur_opcode->op2, &is_var);
 		}
 
 		fse = XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack)));
-		t = xdebug_return_trace_assignment(fse, full_varname, val, op, file, lineno TSRMLS_CC);
+		if (XG(do_trace) && XG(trace_context) && XG(collect_assignments) && XG(trace_handler)->assignment) {
+			XG(trace_handler)->assignment(XG(trace_context), fse, full_varname, val, op, file, lineno TSRMLS_CC);
+		}
 		xdfree(full_varname);
-		fprintf(XG(trace_file), "%s", t);
-		fflush(XG(trace_file));
-		xdfree(t);
 	}
 	return ZEND_USER_OPCODE_DISPATCH;
 }
@@ -284,6 +355,9 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sub,"-=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_mul,"*=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_div,"/=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_mod,"%=",0)
+#if PHP_VERSION_ID >= 50600
+XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_pow,"**=",0)
+#endif
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sl,"<<=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_sr,">>=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(pre_inc,"",0)
@@ -301,7 +375,7 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_xor,"^=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_dim,"=",1)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_obj,"=",1)
 
-void xdebug_count_line(char *filename, int lineno, int executable, int deadcode TSRMLS_DC)
+inline void xdebug_count_line(char *filename, int lineno, int executable, int deadcode TSRMLS_DC)
 {
 	xdebug_coverage_file *file;
 	xdebug_coverage_line *line;
@@ -311,12 +385,9 @@ void xdebug_count_line(char *filename, int lineno, int executable, int deadcode
 	} else {
 		/* Check if the file already exists in the hash */
 		if (!xdebug_hash_find(XG(code_coverage), filename, strlen(filename), (void *) &file)) {
-			/* The file does not exist, so we add it to the hash, and
-			 *  add a line element to the file */
-			file = xdmalloc(sizeof(xdebug_coverage_file));
-			file->name = xdstrdup(filename);
-			file->lines = xdebug_hash_alloc(128, xdebug_coverage_line_dtor);
-		
+			/* The file does not exist, so we add it to the hash */
+			file = xdebug_coverage_file_ctor(filename);
+
 			xdebug_hash_add(XG(code_coverage), filename, strlen(filename), file);
 		}
 		XG(previous_filename) = file->name;
@@ -360,13 +431,10 @@ static void prefill_from_opcode(char *fn, zend_op opcode, int deadcode TSRMLS_DC
 	}
 }
 
-static zend_brk_cont_element* xdebug_find_brk_cont(zval *nest_levels_zval, int array_offset, zend_op_array *op_array)
+static zend_brk_cont_element* xdebug_find_brk_cont(zend_uint nest_levels, int array_offset, zend_op_array *op_array)
 {
-	int nest_levels;
 	zend_brk_cont_element *jmp_to;
 
-	nest_levels = nest_levels_zval->value.lval;
-
 	do {
 		if (array_offset == -1) {
 			/* broken break/continue in code */
@@ -384,7 +452,7 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 
 	zend_op opcode = opa->opcodes[position];
 	if (opcode.opcode == ZEND_JMP) {
-		*jmp1 = ((long) opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+		*jmp1 = ((long) opcode.op1.jmp_addr - (long) base_address) / sizeof(zend_op);
 		return 1;
 	} else if (
 		opcode.opcode == ZEND_JMPZ ||
@@ -393,23 +461,19 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		opcode.opcode == ZEND_JMPNZ_EX
 	) {
 		*jmp1 = position + 1;
-		*jmp2 = ((long) opcode.XDEBUG_ZNODE_ELEM(op2, jmp_addr) - (long) base_address) / sizeof(zend_op);
+		*jmp2 = ((long) opcode.op2.jmp_addr - (long) base_address) / sizeof(zend_op);
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
-		*jmp1 = opcode.XDEBUG_ZNODE_ELEM(op2, opline_num);
+		*jmp1 = opcode.op2.opline_num;
 		*jmp2 = opcode.extended_value;
 		return 1;
 	} else if (opcode.opcode == ZEND_BRK || opcode.opcode == ZEND_CONT) {
 		zend_brk_cont_element *el;
 
-		if (opcode.XDEBUG_TYPE(op2) == IS_CONST
-		    && opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) != (zend_op*) 0xFFFFFFFF
+		if (opcode.op2_type == IS_CONST
+		    && opcode.op1.jmp_addr != (zend_op*) 0xFFFFFFFF
 		) {
-#if PHP_VERSION_ID >= 50399
-			el = xdebug_find_brk_cont(opcode.op2.zv, opcode.XDEBUG_ZNODE_ELEM(op1, opline_num), opa);
-#else
-			el = xdebug_find_brk_cont(&opcode.op2.u.constant, opcode.op1.u.opline_num, opa);
-#endif
+			el = xdebug_find_brk_cont(Z_LVAL_P(opcode.op2.zv), opcode.op1.opline_num, opa);
 			if (el) {
 				*jmp1 = opcode.opcode == ZEND_BRK ? el->brk : el->cont;
 				return 1;
@@ -420,45 +484,76 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		}
 	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
 		*jmp1 = position + 1;
-		*jmp2 = opcode.XDEBUG_ZNODE_ELEM(op2, opline_num);
+		*jmp2 = opcode.op2.opline_num;
+		return 1;
+	} else if (opcode.opcode == ZEND_CATCH) {
+		*jmp1 = position + 1;
+		if (!opcode.result.num) {
+			*jmp2 = opcode.extended_value;
+			if (*jmp2 == *jmp1) {
+				*jmp2 = XDEBUG_JMP_NOT_SET;
+			}
+		} else {
+			*jmp2 = XDEBUG_JMP_EXIT;
+		}
 		return 1;
-#if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
 	} else if (opcode.opcode == ZEND_GOTO) {
-		*jmp1 = ((long) opcode.XDEBUG_ZNODE_ELEM(op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+		*jmp1 = ((long) opcode.op1.jmp_addr - (long) base_address) / sizeof(zend_op);
+		return 1;
+	} else if (opcode.opcode == ZEND_EXIT || opcode.opcode == ZEND_THROW || opcode.opcode == ZEND_RETURN) {
+		*jmp1 = XDEBUG_JMP_EXIT;
 		return 1;
-#endif
 	}
 	return 0;
 }
 
-static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xdebug_set *set TSRMLS_DC)
+static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xdebug_set *set, xdebug_branch_info *branch_info TSRMLS_DC)
 {
-	long jump_pos1;
-	long jump_pos2;
+	long jump_pos1 = XDEBUG_JMP_NOT_SET;
+	long jump_pos2 = XDEBUG_JMP_NOT_SET;
 
 	/*(fprintf(stderr, "Branch analysis from position: %d\n", position);)*/
+
+	if (branch_info) {
+		xdebug_set_add(branch_info->starts, position);
+		branch_info->branches[position].start_lineno = opa->opcodes[position].lineno;
+	}
+
 	/* First we see if the branch has been visited, if so we bail out. */
 	if (xdebug_set_in(set, position)) {
 		return;
 	}
+
+	/*(fprintf(stderr, "XDEBUG Adding %d\n", position);)*/
 	/* Loop over the opcodes until the end of the array, or until a jump point has been found */
 	xdebug_set_add(set, position);
-	/*(fprintf(stderr, "XDEBUG Adding %d\n", position);)*/
 	while (position < opa->last) {
-		jump_pos1 = -1;
-		jump_pos2 = -1;
+		jump_pos1 = XDEBUG_JMP_NOT_SET;
+		jump_pos2 = XDEBUG_JMP_NOT_SET;
 
 		/* See if we have a jump instruction */
 		if (xdebug_find_jump(opa, position, &jump_pos1, &jump_pos2)) {
 			/*(fprintf(stderr, "XDEBUG Jump found. Position 1 = %d", jump_pos1);)*/
-			if (jump_pos2 != -1) {
+			if (jump_pos2 != XDEBUG_JMP_NOT_SET) {
 				/*(fprintf(stderr, ", Position 2 = %d\n", jump_pos2))*/;
 			} else {
 				/*(fprintf(stderr, "\n"))*/;
 			}
-			xdebug_analyse_branch(opa, jump_pos1, set TSRMLS_CC);
-			if (jump_pos2 != -1 && jump_pos2 <= opa->last) {
-				xdebug_analyse_branch(opa, jump_pos2, set TSRMLS_CC);
+			if (jump_pos1 == XDEBUG_JMP_EXIT || jump_pos1 != XDEBUG_JMP_NOT_SET) {
+				if (branch_info) {
+					xdebug_branch_info_update(branch_info, position, opa->opcodes[position].lineno, 0, jump_pos1);
+				}
+				if (jump_pos1 != XDEBUG_JMP_EXIT) {
+					xdebug_analyse_branch(opa, jump_pos1, set, branch_info TSRMLS_CC);
+				}
+			}
+			if (jump_pos2 == XDEBUG_JMP_EXIT || jump_pos2 != XDEBUG_JMP_NOT_SET) {
+				if (branch_info) {
+					xdebug_branch_info_update(branch_info, position, opa->opcodes[position].lineno, 1, jump_pos2);
+				}
+				if (jump_pos2 != XDEBUG_JMP_EXIT) {
+					xdebug_analyse_branch(opa, jump_pos2, set, branch_info TSRMLS_CC);
+				}
 			}
 			break;
 		}
@@ -466,22 +561,32 @@ static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xde
 		/* See if we have a throw instruction */
 		if (opa->opcodes[position].opcode == ZEND_THROW) {
 			/* fprintf(stderr, "Throw found at %d\n", position); */
+			if (branch_info) {
+				xdebug_set_add(branch_info->ends, position);
+				branch_info->branches[position].start_lineno = opa->opcodes[position].lineno;
+			}
 			break;
 		}
 
 		/* See if we have an exit instruction */
 		if (opa->opcodes[position].opcode == ZEND_EXIT) {
 			/* fprintf(stderr, "X* Return found\n"); */
+			if (branch_info) {
+				xdebug_set_add(branch_info->ends, position);
+				branch_info->branches[position].start_lineno = opa->opcodes[position].lineno;
+			}
 			break;
 		}
 		/* See if we have a return instruction */
 		if (
 			opa->opcodes[position].opcode == ZEND_RETURN
-#if PHP_VERSION_ID >= 50400
 			|| opa->opcodes[position].opcode == ZEND_RETURN_BY_REF
-#endif
 		) {
 			/*(fprintf(stderr, "XDEBUG Return found\n");)*/
+			if (branch_info) {
+				xdebug_set_add(branch_info->ends, position);
+				branch_info->branches[position].start_lineno = opa->opcodes[position].lineno;
+			}
 			break;
 		}
 
@@ -491,65 +596,134 @@ static void xdebug_analyse_branch(zend_op_array *opa, unsigned int position, xde
 	}
 }
 
-static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set TSRMLS_DC)
+static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set, xdebug_branch_info *branch_info TSRMLS_DC)
 {
 	unsigned int position = 0;
 
 	while (position < opa->last) {
 		if (position == 0) {
-			xdebug_analyse_branch(opa, position, set TSRMLS_CC);
+			xdebug_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			if (branch_info) {
+				xdebug_set_add(branch_info->entry_points, position);
+			}
 		} else if (opa->opcodes[position].opcode == ZEND_CATCH) {
-			xdebug_analyse_branch(opa, position, set TSRMLS_CC);
+			xdebug_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			if (branch_info) {
+				xdebug_set_add(branch_info->entry_points, position);
+			}
 		}
 		position++;
 	}
+	if (branch_info) {
+		xdebug_set_add(branch_info->ends, opa->last-1);
+		branch_info->branches[opa->last-1].start_lineno = opa->opcodes[opa->last-1].lineno;
+	}
 }
 
-static void prefill_from_oparray(char *fn, zend_op_array *op_array TSRMLS_DC)
+static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa TSRMLS_DC)
+{
+	int closure = 0;
+
+	memset(tmp, 0, sizeof(xdebug_func));
+
+	if (opa->function_name) {
+		if (strcmp(opa->function_name, "{closure}") == 0) {
+			tmp->function = xdebug_sprintf(
+				"{closure:%s:%d-%d}",
+				opa->filename,
+				opa->line_start,
+				opa->line_end
+			);
+			closure = 1;
+		} else {
+			tmp->function = xdstrdup(opa->function_name);
+		}
+	} else {
+		tmp->function = xdstrdup("{main}");
+	}
+
+	if (opa->scope && !closure) {
+		tmp->type = XFUNC_MEMBER;
+		tmp->class = xdstrdup(opa->scope->name);
+	} else {
+		tmp->type = XFUNC_NORMAL;
+	}
+}
+
+static char* xdebug_func_format(xdebug_func *func TSRMLS_DC)
+{
+	switch (func->type) {
+		case XFUNC_NORMAL:
+			return xdstrdup(func->function);
+		case XFUNC_MEMBER:
+			return xdebug_sprintf("%s->%s", func->class, func->function);
+		default:
+			return xdstrdup("???");
+	}
+}
+
+static void prefill_from_oparray(char *filename, zend_op_array *op_array TSRMLS_DC)
 {
 	unsigned int i;
 	xdebug_set *set = NULL;
+	xdebug_branch_info *branch_info = NULL;
 
-	op_array->reserved[XG(reserved_offset)] = (void*) 1;
+	op_array->reserved[XG(dead_code_analysis_tracker_offset)] = (void*) XG(dead_code_last_start_id);
 
 	/* Check for abstract methods and simply return from this function in those
 	 * cases. */
-#if PHP_VERSION_ID >= 50300
-	if (op_array->last >= 3 && op_array->opcodes[op_array->last - 3].opcode == ZEND_RAISE_ABSTRACT_ERROR)
-#else
-	if (op_array->last >= 4 && op_array->opcodes[op_array->last - 4].opcode == ZEND_RAISE_ABSTRACT_ERROR)
-#endif
-	{
+	if (op_array->last >= 3 && op_array->opcodes[op_array->last - 3].opcode == ZEND_RAISE_ABSTRACT_ERROR) {
 		return;
 	}	
 
 	/* Run dead code analysis if requested */
-	if (XG(code_coverage_dead_code_analysis) && XDEBUG_PASS_TWO_DONE) {
+	if (XG(code_coverage_dead_code_analysis) && (op_array->fn_flags & ZEND_ACC_DONE_PASS_TWO)) {
 		set = xdebug_set_create(op_array->last);
-		xdebug_analyse_oparray(op_array, set TSRMLS_CC);
+		if (XG(code_coverage_branch_check)) {
+			branch_info = xdebug_branch_info_create(op_array->last);
+		}
+
+		xdebug_analyse_oparray(op_array, set, branch_info TSRMLS_CC);
 	}
 
 	/* The normal loop then finally */
 	for (i = 0; i < op_array->last; i++) {
 		zend_op opcode = op_array->opcodes[i];
-		prefill_from_opcode(fn, opcode, set ? !xdebug_set_in(set, i) : 0 TSRMLS_CC);
+		prefill_from_opcode(filename, opcode, set ? !xdebug_set_in(set, i) : 0 TSRMLS_CC);
 	}
 
 	if (set) {
 		xdebug_set_free(set);
 	}
+	if (branch_info) {
+		char *function_name;
+		xdebug_func func_info;
+
+		xdebug_build_fname_from_oparray(&func_info, op_array TSRMLS_CC);
+		function_name = xdebug_func_format(&func_info TSRMLS_CC);
+
+		if (func_info.class) {
+			xdfree(func_info.class);
+		}
+		if (func_info.function) {
+			xdfree(func_info.function);
+		}
+
+		xdebug_branch_post_process(op_array, branch_info);
+		xdebug_branch_find_paths(branch_info);
+		xdebug_branch_info_add_branches_and_paths(filename, (char*) function_name, branch_info TSRMLS_CC);
+
+		xdfree(function_name);
+	}
 }
 
-static int prefill_from_function_table(zend_op_array *opa XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+static int prefill_from_function_table(zend_op_array *opa TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	char *new_filename;
-#if !defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300
-	TSRMLS_FETCH();
-#endif
 
 	new_filename = va_arg(args, char*);
 	if (opa->type == ZEND_USER_FUNCTION) {
-		if (opa->reserved[XG(reserved_offset)] != (void*) 1 /* && opa->filename && strcmp(opa->filename, new_filename) == 0)*/) {
+		if ((long) opa->reserved[XG(dead_code_analysis_tracker_offset)] < XG(dead_code_last_start_id)) {
 			prefill_from_oparray((char *) opa->filename, opa TSRMLS_CC);
 		}
 	}
@@ -557,7 +731,7 @@ static int prefill_from_function_table(zend_op_array *opa XDEBUG_ZEND_HASH_APPLY
 	return ZEND_HASH_APPLY_KEEP;
 }
 
-static int prefill_from_class_table(zend_class_entry **class_entry XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+static int prefill_from_class_table(zend_class_entry **class_entry TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	char *new_filename;
 	zend_class_entry *ce;
@@ -568,7 +742,7 @@ static int prefill_from_class_table(zend_class_entry **class_entry XDEBUG_ZEND_H
 	if (ce->type == ZEND_USER_CLASS) {
 		if (!(ce->ce_flags & ZEND_XDEBUG_VISITED)) {
 			ce->ce_flags |= ZEND_XDEBUG_VISITED;
-			zend_hash_apply_with_arguments(&ce->function_table XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, new_filename);
+			zend_hash_apply_with_arguments(&ce->function_table TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, new_filename);
 		}
 	}
 
@@ -577,12 +751,61 @@ static int prefill_from_class_table(zend_class_entry **class_entry XDEBUG_ZEND_H
 
 void xdebug_prefill_code_coverage(zend_op_array *op_array TSRMLS_DC)
 {
-	if (op_array->reserved[XG(reserved_offset)] != (void*) 1) {
+	if ((long) op_array->reserved[XG(dead_code_analysis_tracker_offset)] < XG(dead_code_last_start_id)) {
 		prefill_from_oparray((char *) op_array->filename, op_array TSRMLS_CC);
 	}
 
-	zend_hash_apply_with_arguments(CG(function_table)  XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, op_array->filename);
-	zend_hash_apply_with_arguments(CG(class_table) XDEBUG_ZEND_HASH_APPLY_TSRMLS_CC, (apply_func_args_t) prefill_from_class_table, 1, op_array->filename);
+	zend_hash_apply_with_arguments(CG(function_table)  TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, op_array->filename);
+	zend_hash_apply_with_arguments(CG(class_table) TSRMLS_CC, (apply_func_args_t) prefill_from_class_table, 1, op_array->filename);
+}
+
+void xdebug_code_coverage_start_of_function(zend_op_array *op_array TSRMLS_DC)
+{
+	xdebug_path *path = xdebug_path_new(NULL);
+
+	xdebug_prefill_code_coverage(op_array TSRMLS_CC);
+	xdebug_path_info_add_path_for_level(XG(paths_stack), path, XG(level) TSRMLS_CC);
+
+	if (XG(branches).size == 0 || XG(level) >= XG(branches).size) {
+		XG(branches).size = XG(level) + 32;
+		XG(branches).last_branch_nr = realloc(XG(branches).last_branch_nr, sizeof(int) * XG(branches.size));
+	}
+
+	XG(branches).last_branch_nr[XG(level)] = -1;
+}
+
+void xdebug_code_coverage_end_of_function(zend_op_array *op_array TSRMLS_DC)
+{
+	xdebug_str str = { 0, 0, NULL };
+	xdebug_path *path = xdebug_path_info_get_path_for_level(XG(paths_stack), XG(level) TSRMLS_CC);
+	char *file = (char *) op_array->filename;
+	xdebug_func func_info;
+	char *function_name;
+
+	if (!path) {
+		return;
+	}
+
+	xdebug_create_key_for_path(path, &str);
+
+	xdebug_build_fname_from_oparray(&func_info, op_array TSRMLS_CC);
+	function_name = xdebug_func_format(&func_info TSRMLS_CC);
+
+	if (func_info.class) {
+		xdfree(func_info.class);
+	}
+	if (func_info.function) {
+		xdfree(func_info.function);
+	}
+
+	xdebug_branch_info_mark_end_of_function_reached(file, function_name, str.d, str.l TSRMLS_CC);
+
+	xdfree(function_name);
+	xdfree(str.d);
+
+	if (path) {
+		xdebug_path_free(path);
+	}
 }
 
 PHP_FUNCTION(xdebug_start_code_coverage)
@@ -594,6 +817,7 @@ PHP_FUNCTION(xdebug_start_code_coverage)
 	}
 	XG(code_coverage_unused) = (options & XDEBUG_CC_OPTION_UNUSED);
 	XG(code_coverage_dead_code_analysis) = (options & XDEBUG_CC_OPTION_DEAD_CODE);
+	XG(code_coverage_branch_check) = (options & XDEBUG_CC_OPTION_BRANCH_CHECK);
 
 	if (!XG(extended_info)) {
 		php_error(E_WARNING, "You can only use code coverage when you leave the setting of 'xdebug.extended_info' to the default '1'.");
@@ -618,8 +842,13 @@ PHP_FUNCTION(xdebug_stop_code_coverage)
 		if (cleanup) {
 			XG(previous_filename) = "";
 			XG(previous_file) = NULL;
+			XG(previous_mark_filename) = "";
+			XG(previous_mark_file) = NULL;
 			xdebug_hash_destroy(XG(code_coverage));
 			XG(code_coverage) = xdebug_hash_alloc(32, xdebug_coverage_file_dtor);
+			XG(dead_code_last_start_id)++;
+			xdebug_path_info_dtor(XG(paths_stack));
+			XG(paths_stack) = xdebug_path_info_ctor();
 		}
 		XG(do_code_coverage) = 0;
 		RETURN_TRUE;
@@ -655,25 +884,133 @@ static void add_line(void *ret, xdebug_hash_element *e)
 	}
 }
 
+static void add_branches(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC)
+{
+	zval *branches, *branch, *out, *out_hit;
+	unsigned int i;
+
+	MAKE_STD_ZVAL(branches);
+	array_init(branches);
+
+	for (i = 0; i < branch_info->starts->size; i++) {
+		if (xdebug_set_in(branch_info->starts, i)) {
+			MAKE_STD_ZVAL(branch);
+			array_init(branch);
+			add_assoc_long(branch, "op_start", i);
+			add_assoc_long(branch, "op_end", branch_info->branches[i].end_op);
+			add_assoc_long(branch, "line_start", branch_info->branches[i].start_lineno);
+			add_assoc_long(branch, "line_end", branch_info->branches[i].end_lineno);
+
+			add_assoc_long(branch, "hit", branch_info->branches[i].hit);
+
+			MAKE_STD_ZVAL(out);
+			array_init(out);
+			if (branch_info->branches[i].out[0]) {
+				add_index_long(out, 0, branch_info->branches[i].out[0]);
+			}
+			if (branch_info->branches[i].out[1]) {
+				add_index_long(out, 1, branch_info->branches[i].out[1]);
+			}
+			add_assoc_zval(branch, "out", out);
+
+			MAKE_STD_ZVAL(out_hit);
+			array_init(out_hit);
+			if (branch_info->branches[i].out[0]) {
+				add_index_long(out_hit, 0, branch_info->branches[i].out_hit[0]);
+			}
+			if (branch_info->branches[i].out[1]) {
+				add_index_long(out_hit, 1, branch_info->branches[i].out_hit[1]);
+			}
+			add_assoc_zval(branch, "out_hit", out_hit);
+
+			add_index_zval(branches, i, branch);
+		}
+	}
+
+	add_assoc_zval_ex(retval, "branches", 9, branches);
+}
+
+static void add_paths(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC)
+{
+	zval *paths, *path, *path_container;
+	unsigned int i, j;
+
+	MAKE_STD_ZVAL(paths);
+	array_init(paths);
+
+	for (i = 0; i < branch_info->path_info.paths_count; i++) {
+		MAKE_STD_ZVAL(path);
+		array_init(path);
+
+		MAKE_STD_ZVAL(path_container);
+		array_init(path_container);
+
+		for (j = 0; j < branch_info->path_info.paths[i]->elements_count; j++) {
+			add_next_index_long(path, branch_info->path_info.paths[i]->elements[j]);
+		}
+
+		add_assoc_zval(path_container, "path", path);
+		add_assoc_long(path_container, "hit", branch_info->path_info.paths[i]->hit);
+
+		add_next_index_zval(paths, path_container);
+	}
+
+	add_assoc_zval_ex(retval, "paths", 6, paths);
+}
+
+static void add_cc_function(void *ret, xdebug_hash_element *e)
+{
+	xdebug_coverage_function *function = (xdebug_coverage_function*) e->ptr;
+	zval                     *retval = (zval*) ret;
+	zval                     *function_info;
+	TSRMLS_FETCH();
+
+	MAKE_STD_ZVAL(function_info);
+	array_init(function_info);
+
+	if (function->branch_info) {
+		add_branches(function_info, function->branch_info TSRMLS_CC);
+		add_paths(function_info, function->branch_info TSRMLS_CC);
+	}
+
+	add_assoc_zval_ex(retval, function->name, strlen(function->name) + 1, function_info);
+}
+
 static void add_file(void *ret, xdebug_hash_element *e)
 {
 	xdebug_coverage_file *file = (xdebug_coverage_file*) e->ptr;
 	zval                 *retval = (zval*) ret;
-	zval                 *lines;
+	zval                 *lines, *functions, *file_info;
 	HashTable            *target_hash;
 	TSRMLS_FETCH();
 
+	/* Add all the lines */
 	MAKE_STD_ZVAL(lines);
 	array_init(lines);
 
-	/* Add all the lines */
 	xdebug_hash_apply(file->lines, (void *) lines, add_line);
 
 	/* Sort on linenumber */
 	target_hash = HASH_OF(lines);
 	zend_hash_sort(target_hash, zend_qsort, xdebug_lineno_cmp, 0 TSRMLS_CC);
 
-	add_assoc_zval_ex(retval, file->name, strlen(file->name) + 1, lines);
+	/* Add the branch and path info */
+	if (file->has_branch_info) {
+		MAKE_STD_ZVAL(file_info);
+		array_init(file_info);
+
+		MAKE_STD_ZVAL(functions);
+		array_init(functions);
+
+		xdebug_hash_apply(file->functions, (void *) functions, add_cc_function);
+
+		add_assoc_zval_ex(file_info, "lines", 6, lines);
+		add_assoc_zval_ex(file_info, "functions", 10, functions);
+
+		add_assoc_zval_ex(retval, file->name, strlen(file->name) + 1, file_info);
+	} else {
+		add_assoc_zval_ex(retval, file->name, strlen(file->name) + 1, lines);
+	}
 }
 
 PHP_FUNCTION(xdebug_get_code_coverage)
@@ -686,3 +1023,8 @@ PHP_FUNCTION(xdebug_get_function_count)
 {
 	RETURN_LONG(XG(function_count));
 }
+
+PHP_FUNCTION(xdebug_code_coverage_started)
+{
+	RETURN_BOOL(XG(do_code_coverage));
+}
