@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -22,6 +22,7 @@
 #include "php.h"
 #include "xdebug_hash.h"
 #include "xdebug_mm.h"
+#include "xdebug_branch_info.h"
 
 typedef struct xdebug_coverage_line {
 	int lineno;
@@ -30,10 +31,17 @@ typedef struct xdebug_coverage_line {
 } xdebug_coverage_line;
 
 typedef struct xdebug_coverage_file {
-	char        *name;
-	xdebug_hash *lines;
+	char               *name;
+	xdebug_hash        *lines;
+	xdebug_hash        *functions; /* Used for branch coverage */
+	int                 has_branch_info;
 } xdebug_coverage_file;
 
+typedef struct xdebug_coverage_function {
+	char               *name;
+	xdebug_branch_info *branch_info;
+} xdebug_coverage_function;
+
 /* Needed for code coverage as Zend doesn't always add EXT_STMT when expected */
 #define XDEBUG_SET_OPCODE_OVERRIDE_COMMON(oc) \
 	zend_set_user_opcode_handler(oc, xdebug_common_override_handler);
@@ -42,8 +50,17 @@ typedef struct xdebug_coverage_file {
 
 
 void xdebug_coverage_line_dtor(void *data);
+
+xdebug_coverage_file *xdebug_coverage_file_ctor(char *filename);
 void xdebug_coverage_file_dtor(void *data);
 
+xdebug_coverage_function *xdebug_coverage_function_ctor(char *function_name);
+void xdebug_coverage_function_dtor(void *data);
+void xdebug_print_opcode_info(char type, zend_execute_data *execute_data, zend_op *cur_opcode TSRMLS_DC);
+void xdebug_code_coverage_start_of_function(zend_op_array *op_array TSRMLS_DC);
+void xdebug_code_coverage_end_of_function(zend_op_array *op_array TSRMLS_DC);
+
+int xdebug_check_branch_entry_handler(ZEND_OPCODE_HANDLER_ARGS);
 int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS);
 
 #define XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(f) \
@@ -55,6 +72,9 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_sub);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_mul);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_div);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_mod);
+#if PHP_VERSION_ID >= 50600
+XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_pow);
+#endif
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_sl);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(assign_sr);
 XDEBUG_OPCODE_OVERRIDE_ASSIGN_DECL(pre_inc);
@@ -78,6 +98,7 @@ void xdebug_prefill_code_coverage(zend_op_array *op_array TSRMLS_DC);
 PHP_FUNCTION(xdebug_start_code_coverage);
 PHP_FUNCTION(xdebug_stop_code_coverage);
 PHP_FUNCTION(xdebug_get_code_coverage);
+PHP_FUNCTION(xdebug_code_coverage_started);
 
 PHP_FUNCTION(xdebug_get_function_count);
 
