@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -48,6 +48,7 @@ void xdebug_stop_trace(TSRMLS_D);
 typedef struct xdebug_var {
 	char *name;
 	void *addr;
+	int   is_variadic;
 } xdebug_var;
 
 #define XFUNC_UNKNOWN        0x00
@@ -78,12 +79,14 @@ typedef struct xdebug_var {
 
 #define XDEBUG_MAX_FUNCTION_LEN 1024
 
-#define XDEBUG_TRACE_OPTION_APPEND       1
-#define XDEBUG_TRACE_OPTION_COMPUTERIZED 2
-#define XDEBUG_TRACE_OPTION_HTML         4
+#define XDEBUG_TRACE_OPTION_APPEND         1
+#define XDEBUG_TRACE_OPTION_COMPUTERIZED   2
+#define XDEBUG_TRACE_OPTION_HTML           4
+#define XDEBUG_TRACE_OPTION_NAKED_FILENAME 8
 
 #define XDEBUG_CC_OPTION_UNUSED          1
 #define XDEBUG_CC_OPTION_DEAD_CODE       2
+#define XDEBUG_CC_OPTION_BRANCH_CHECK    4
 
 #define STATUS_STARTING   0
 #define STATUS_STOPPING   1
@@ -115,7 +118,8 @@ typedef struct xdebug_var {
 #define XDEBUG_ERROR_EVALUATING_CODE               206
 #define XDEBUG_ERROR_INVALID_EXPRESSION            207 /* unused */
 
-#define XDEBUG_ERROR_PROPERTY_NON_EXISTANT         300
+#define XDEBUG_ERROR_PROPERTY_NON_EXISTENT         300
+#define XDEBUG_ERROR_PROPERTY_NON_EXISTANT         300 /* compatibility typo */
 #define XDEBUG_ERROR_STACK_DEPTH_INVALID           301
 #define XDEBUG_ERROR_CONTEXT_INVALID               302 /* unused */
 
@@ -203,6 +207,23 @@ function_stack_entry *xdebug_get_stack_head(TSRMLS_D);
 function_stack_entry *xdebug_get_stack_frame(int nr TSRMLS_DC);
 function_stack_entry *xdebug_get_stack_tail(TSRMLS_D);
 
+typedef struct
+{
+	void *(*init)(char *fname, long options TSRMLS_DC);
+	void (*deinit)(void *ctxt TSRMLS_DC);
+	void (*write_header)(void *ctxt TSRMLS_DC);
+	void (*write_footer)(void *ctxt TSRMLS_DC);
+	char *(*get_filename)(void *ctxt TSRMLS_DC);
+	void (*function_entry)(void *ctxt, function_stack_entry *fse, int function_nr TSRMLS_DC);
+	void (*function_exit)(void *ctxt, function_stack_entry *fse, int function_nr TSRMLS_DC);
+	void (*return_value)(void *ctxt, function_stack_entry *fse, int function_nr, zval *return_value TSRMLS_DC);
+#if PHP_VERSION_ID >= 50500
+	void (*generator_return_value)(void *ctxt, function_stack_entry *fse, int function_nr, zend_generator *generator TSRMLS_DC);
+#endif
+	void (*assignment)(void *ctxt, function_stack_entry *fse, char *full_varname, zval *value, char *op, char *file, int lineno TSRMLS_DC);
+} xdebug_trace_handler_t;
+
+
 xdebug_hash* xdebug_used_var_hash_from_llist(xdebug_llist *list);
 void xdebug_init_debugger(TSRMLS_D);
 
