@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -25,6 +25,7 @@
 #include "xdebug_superglobals.h"
 #include "xdebug_var.h"
 #include "ext/standard/html.h"
+#include "ext/standard/php_smart_str.h"
 
 #include "main/php_ini.h"
 
@@ -34,11 +35,7 @@ static char* text_formats[11] = {
 	"\n",
 	"%s: %s in %s on line %d\n",
 	"\nCall Stack:\n",
-#if HAVE_PHP_MEMORY_USAGE
-	"%10.4f %10ld %3d. %s(",
-#else
-	"%10.4f %3d. %s(",
-#endif
+	"%10.4F %10ld %3d. %s(",
 	"'%s'",
 	") %s:%d\n",
 	"\n\nVariables in local scope (#%d):\n",
@@ -52,11 +49,7 @@ static char* ansi_formats[11] = {
 	"\n",
 	"[1m[31m%s[0m: %s[22m in [31m%s[0m on line [32m%d[0m[22m\n",
 	"\n[1mCall Stack:[22m\n",
-#if HAVE_PHP_MEMORY_USAGE
-	"%10.4f %10ld %3d. %s(",
-#else
-	"%10.4f %3d. %s(",
-#endif
+	"%10.4F %10ld %3d. %s(",
 	"'%s'",
 	") %s:%d\n",
 	"\n\nVariables in local scope (#%d):\n",
@@ -69,24 +62,15 @@ static char* ansi_formats[11] = {
 static char* html_formats[13] = {
 	"<br />\n<font size='1'><table class='xdebug-error xe-%s%s' dir='ltr' border='1' cellspacing='0' cellpadding='1'>\n",
 	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> %s: %s in %s on line <i>%d</i></th></tr>\n",
-#if HAVE_PHP_MEMORY_USAGE
 	"<tr><th align='left' bgcolor='#e9b96e' colspan='5'>Call Stack</th></tr>\n<tr><th align='center' bgcolor='#eeeeec'>#</th><th align='left' bgcolor='#eeeeec'>Time</th><th align='left' bgcolor='#eeeeec'>Memory</th><th align='left' bgcolor='#eeeeec'>Function</th><th align='left' bgcolor='#eeeeec'>Location</th></tr>\n",
-	"<tr><td bgcolor='#eeeeec' align='center'>%d</td><td bgcolor='#eeeeec' align='center'>%.4f</td><td bgcolor='#eeeeec' align='right'>%ld</td><td bgcolor='#eeeeec'>%s( ",
-#else
-	"<tr><th align='left' bgcolor='#e9b96e' colspan='4'>Call Stack</th></tr>\n<tr><th align='center' bgcolor='#eeeeec'>#</th><th align='left' bgcolor='#eeeeec'>Time</th><th align='left' bgcolor='#eeeeec'>Function</th><th align='left' bgcolor='#eeeeec'>Location</th></tr>\n",
-	"<tr><td bgcolor='#eeeeec' align='center'>%d</td><td bgcolor='#eeeeec' align='center'>%.4f</td><td bgcolor='#eeeeec'>%s( ",
-#endif
+	"<tr><td bgcolor='#eeeeec' align='center'>%d</td><td bgcolor='#eeeeec' align='center'>%.4F</td><td bgcolor='#eeeeec' align='right'>%ld</td><td bgcolor='#eeeeec'>%s( ",
 	"<font color='#00bb00'>'%s'</font>",
-	" )</td><td title='%s' bgcolor='#eeeeec'>..%s<b>:</b>%d</td></tr>\n",
-#if HAVE_PHP_MEMORY_USAGE
+	" )</td><td title='%s' bgcolor='#eeeeec'>...%s<b>:</b>%d</td></tr>\n",
 	"<tr><th align='left' colspan='5' bgcolor='#e9b96e'>Variables in local scope (#%d)</th></tr>\n",
-#else
-	"<tr><th align='left' colspan='4' bgcolor='#e9b96e'>Variables in local scope (#%d)</th></tr>\n",
-#endif
 	"</table></font>\n",
 	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec'>%s</td></tr>\n",
 	"<tr><td colspan='2' align='right' bgcolor='#eeeeec' valign='top'><pre>$%s&nbsp;=</pre></td><td colspan='3' bgcolor='#eeeeec' valign='top'><i>Undefined</i></td></tr>\n",
-	" )</td><td title='%s' bgcolor='#eeeeec'><a style='color: black' href='%s'>..%s<b>:</b>%d</a></td></tr>\n",
+	" )</td><td title='%s' bgcolor='#eeeeec'><a style='color: black' href='%s'>...%s<b>:</b>%d</a></td></tr>\n",
 	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> %s: %s in <a style='color: black' href='%s'>%s</a> on line <i>%d</i></th></tr>\n",
 	"<tr><th align='left' bgcolor='#f57900' colspan=\"5\"><span style='background-color: #cc0000; color: #fce94f; font-size: x-large;'>( ! )</span> SCREAM: Error suppression ignored for</th></tr>\n"
 };
@@ -124,15 +108,13 @@ static void dump_used_var_with_contents(void *htmlq, xdebug_hash_element* he, vo
 		return;
 	}
 
-#if PHP_VERSION_ID >= 50300
 	if (!EG(active_symbol_table)) {
 		zend_rebuild_symbol_table(TSRMLS_C);
 	}
-#endif
 
 	tmp_ht = XG(active_symbol_table);
 	XG(active_symbol_table) = EG(active_symbol_table);
-	zvar = xdebug_get_php_symbol(name, strlen(name) + 1);
+	zvar = xdebug_get_php_symbol(name, strlen(name) + 1 TSRMLS_CC);
 	XG(active_symbol_table) = tmp_ht;
 
 	formats = select_formats(PG(html_errors) TSRMLS_CC);
@@ -176,6 +158,7 @@ void xdebug_log_stack(const char *error_type_str, char *buffer, const char *erro
 			unsigned int j = 0; /* Counter */
 			char *tmp_name;
 			xdebug_str log_buffer = {0, 0, NULL};
+			int variadic_opened = 0;
 
 			i = XDEBUG_LLIST_VALP(le);
 			tmp_name = xdebug_show_fname(i->function, 0, 0 TSRMLS_CC);
@@ -191,10 +174,20 @@ void xdebug_log_stack(const char *error_type_str, char *buffer, const char *erro
 				} else {
 					c = 1;
 				}
+
+				if (i->var[j].is_variadic && XG(collect_params) != 5) {
+					xdebug_str_add(&log_buffer, "...", 0);
+					variadic_opened = 1;
+				}
+
 				tmp_varname = i->var[j].name ? xdebug_sprintf("$%s = ", i->var[j].name) : xdstrdup("");
 				xdebug_str_add(&log_buffer, tmp_varname, 0);
 				xdfree(tmp_varname);
 
+				if (i->var[j].is_variadic) {
+					xdebug_str_add(&log_buffer, "variadic(", 0);
+				}
+
 				if (i->var[j].addr) {
 					tmp_value = xdebug_get_zval_value(i->var[j].addr, 0, NULL);
 					xdebug_str_add(&log_buffer, tmp_value, 0);
@@ -204,6 +197,10 @@ void xdebug_log_stack(const char *error_type_str, char *buffer, const char *erro
 				}
 			}
 
+			if (variadic_opened) {
+				xdebug_str_add(&log_buffer, ")", 0);
+			}
+
 			xdebug_str_add(&log_buffer, xdebug_sprintf(") %s:%d", i->filename, i->lineno), 1);
 			php_log_err(log_buffer.d TSRMLS_CC);
 			xdebug_str_free(&log_buffer);
@@ -211,40 +208,6 @@ void xdebug_log_stack(const char *error_type_str, char *buffer, const char *erro
 	}
 }
 
-static int create_file_link(char **filename, const char *error_filename, int error_lineno TSRMLS_DC)
-{
-	xdebug_str fname = {0, 0, NULL};
-	char      *format = XG(file_link_format);
-	
-	while (*format)
-	{
-		if (*format != '%') {
-			xdebug_str_addl(&fname, (char *) format, 1, 0);
-		} else {
-			format++;
-			switch (*format)
-			{
-				case 'f': /* filename */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", error_filename), 1);
-					break;
-
-				case 'l': /* line number */
-					xdebug_str_add(&fname, xdebug_sprintf("%d", error_lineno), 1);
-					break;
-
-				case '%': /* literal % */
-					xdebug_str_addl(&fname, "%", 1, 0);
-					break;
-			}
-		}
-		format++;
-	}
-	
-	*filename = fname.d;
-
-	return fname.l;
-}
-
 void xdebug_append_error_head(xdebug_str *str, int html, char *error_type_str TSRMLS_DC)
 {
 	char **formats = select_formats(html TSRMLS_CC);
@@ -266,14 +229,36 @@ void xdebug_append_error_description(xdebug_str *str, int html, const char *erro
 {
 	char **formats = select_formats(html TSRMLS_CC);
 	char *escaped;
-#if PHP_VERSION_ID >= 50400
 	size_t newlen;
-#else
-	int    newlen;
-#endif
 
 	if (html) {
-		escaped = php_escape_html_entities((unsigned char *) buffer, strlen(buffer), &newlen, 0, 0, NULL TSRMLS_CC);
+		char *tmp;
+		char *first_closing = strchr(buffer, ']');
+
+		/* We do need to escape HTML entities here, as HTML chars could be in
+		 * the error message. However, PHP in some circumstances also adds an
+		 * HTML link to a manual page. That bit, we don't need to escape. So
+		 * this bit of code finds the portion that doesn't need escaping, adds
+		 * it to a tmp string, and then adds an HTML escaped string for the
+		 * rest of the original buffer. */
+		if (first_closing && strstr(buffer, "() [<a href=") != NULL) {
+			smart_str special_escaped = {0};
+
+			*first_closing = '\0';
+			first_closing++;
+			smart_str_appends(&special_escaped, buffer);
+
+			tmp = php_escape_html_entities((unsigned char *) first_closing, strlen(first_closing), &newlen, 0, 0, NULL TSRMLS_CC);
+			smart_str_appends(&special_escaped, tmp);
+			STR_FREE(tmp);
+
+			smart_str_0(&special_escaped);
+
+			escaped = estrdup(special_escaped.c);
+			smart_str_free(&special_escaped);
+		} else {
+			escaped = php_escape_html_entities((unsigned char *) buffer, strlen(buffer), &newlen, 0, 0, NULL TSRMLS_CC);
+		}
 	} else {
 		escaped = estrdup(buffer);
 	}
@@ -281,20 +266,21 @@ void xdebug_append_error_description(xdebug_str *str, int html, const char *erro
 	if (strlen(XG(file_link_format)) > 0 && html) {
 		char *file_link;
 
-		create_file_link(&file_link, error_filename, error_lineno TSRMLS_CC);
+		xdebug_format_file_link(&file_link, error_filename, error_lineno TSRMLS_CC);
 		xdebug_str_add(str, xdebug_sprintf(formats[11], error_type_str, escaped, file_link, error_filename, error_lineno), 1);
 		xdfree(file_link);
 	} else {
 		xdebug_str_add(str, xdebug_sprintf(formats[1], error_type_str, escaped, error_filename, error_lineno), 1);
 	}
 
-	efree(escaped);
+	STR_FREE(escaped);
 }
 
 void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 {
 	xdebug_llist_element *le;
 	function_stack_entry *i;
+	int    printed_frames = 0;
 	int    len;
 	char **formats = select_formats(html TSRMLS_CC);
 
@@ -308,27 +294,20 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 			int c = 0; /* Comma flag */
 			unsigned int j = 0; /* Counter */
 			char *tmp_name;
+			int variadic_opened = 0;
 			
 			i = XDEBUG_LLIST_VALP(le);
 			tmp_name = xdebug_show_fname(i->function, html, 0 TSRMLS_CC);
 			if (html) {
-#if HAVE_PHP_MEMORY_USAGE
 				xdebug_str_add(str, xdebug_sprintf(formats[3], i->level, i->time - XG(start_time), i->memory, tmp_name), 1);
-#else
-				xdebug_str_add(str, xdebug_sprintf(formats[3], i->level, i->time - XG(start_time), tmp_name), 1);
-#endif
 			} else {
-#if HAVE_PHP_MEMORY_USAGE
 				xdebug_str_add(str, xdebug_sprintf(formats[3], i->time - XG(start_time), i->memory, i->level, tmp_name), 1);
-#else
-				xdebug_str_add(str, xdebug_sprintf(formats[3], i->time - XG(start_time), i->level, tmp_name), 1);
-#endif
 			}
 			xdfree(tmp_name);
 
 			/* Printing vars */
 			for (j = 0; j < i->varc; j++) {
-				char *tmp_value, *tmp_fancy_value, *tmp_fancy_synop_value;
+				char *tmp_value, *tmp_fancy_value, *tmp_fancy_synop_value, *tmp_serialized;
 				int newlen;
 
 				if (c) {
@@ -337,7 +316,12 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 					c = 1;
 				}
 
-				if (i->var[j].name && XG(collect_params) >= 4) {
+				if (i->var[j].is_variadic && i->var[j].addr) {
+					xdebug_str_add(str, "...", 0);
+					variadic_opened = 1;
+				}
+
+				if (i->var[j].name && XG(collect_params) == 4) {
 					if (html) {
 						xdebug_str_add(str, xdebug_sprintf("<span>$%s = </span>", i->var[j].name), 1);
 					} else {
@@ -345,11 +329,21 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 					}
 				}
 
+
+				if (i->var[j].is_variadic && i->var[j].addr) {
+					if (html) {
+						xdebug_str_add(str, "<i>variadic</i>(", 0);
+					} else {
+						xdebug_str_add(str, "variadic(", 0);
+					}
+				}
+
 				if (i->var[j].addr) {
 					if (html) {
 						tmp_value = xdebug_get_zval_value(i->var[j].addr, 0, NULL);
 						tmp_fancy_value = xdebug_xmlize(tmp_value, strlen(tmp_value), &newlen);
 						tmp_fancy_synop_value = xdebug_get_zval_synopsis_fancy("", i->var[j].addr, &len, 0, NULL TSRMLS_CC);
+						tmp_serialized = xdebug_get_zval_value_serialized(i->var[j].addr, 0, NULL TSRMLS_CC);
 						switch (XG(collect_params)) {
 							case 1: /* synopsis */
 								xdebug_str_add(str, xdebug_sprintf("<span>%s</span>", tmp_fancy_synop_value), 1);
@@ -358,23 +352,32 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 								xdebug_str_add(str, xdebug_sprintf("<span title='%s'>%s</span>", tmp_fancy_value, tmp_fancy_synop_value), 1);
 								break;
 							case 3: /* full */
+							case 4: /* full (with var_name) */
 							default:
 								xdebug_str_add(str, xdebug_sprintf("<span>%s</span>", tmp_fancy_value), 1);
 								break;
+							case 5: /* serialized */
+								xdebug_str_add(str, xdebug_sprintf("<span>%s</span>", tmp_serialized), 1);
+								break;
 						}
 						xdfree(tmp_value);
 						efree(tmp_fancy_value);
 						xdfree(tmp_fancy_synop_value);
+						xdfree(tmp_serialized);
 					} else {
 						switch (XG(collect_params)) {
 							case 1: /* synopsis */
 							case 2:
 								tmp_value = xdebug_get_zval_synopsis(i->var[j].addr, 0, NULL);
 								break;
-							case 3:
+							case 3: /* full */
+							case 4: /* full (with var_name) */
 							default:
 								tmp_value = xdebug_get_zval_value(i->var[j].addr, 0, NULL);
 								break;
+							case 5: /* serialized */
+								tmp_value = xdebug_get_zval_value_serialized(i->var[j].addr, 0, NULL TSRMLS_CC);
+								break;
 						}
 						if (tmp_value) {
 							xdebug_str_add(str, xdebug_sprintf("%s", tmp_value), 1);
@@ -388,6 +391,10 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 				}
 			}
 
+			if (variadic_opened) {
+				xdebug_str_add(str, ")", 0);
+			}
+
 			if (i->include_filename) {
 				xdebug_str_add(str, xdebug_sprintf(formats[4], i->include_filename), 1);
 			}
@@ -397,7 +404,7 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 					char *just_filename = strrchr(i->filename, DEFAULT_SLASH);
 					char *file_link;
 
-					create_file_link(&file_link, i->filename, i->lineno TSRMLS_CC);
+					xdebug_format_file_link(&file_link, i->filename, i->lineno TSRMLS_CC);
 					xdebug_str_add(str, xdebug_sprintf(formats[10], i->filename, file_link, just_filename, i->lineno), 1);
 					xdfree(file_link);
 				} else {
@@ -408,6 +415,11 @@ void xdebug_append_printable_stack(xdebug_str *str, int html TSRMLS_DC)
 			} else {
 				xdebug_str_add(str, xdebug_sprintf(formats[5], i->filename, i->lineno), 1);
 			}
+
+			printed_frames++;
+			if (XG(max_stack_frames) > 0 && printed_frames >= XG(max_stack_frames)) {
+				break;
+			}
 		}
 
 		if (XG(dump_globals) && !(XG(dump_once) && XG(dumped))) {
@@ -446,7 +458,7 @@ void xdebug_append_error_footer(xdebug_str *str, int html TSRMLS_DC)
 	xdebug_str_add(str, formats[7], 0);
 }
 
-static char *get_printable_stack(int html, int error_type, char *buffer, const char *error_filename, const int error_lineno TSRMLS_DC)
+static char *get_printable_stack(int html, int error_type, char *buffer, const char *error_filename, const int error_lineno, int include_decription TSRMLS_DC)
 {
 	char *prepend_string;
 	char *append_string;
@@ -459,7 +471,9 @@ static char *get_printable_stack(int html, int error_type, char *buffer, const c
 
 	xdebug_str_add(&str, prepend_string ? prepend_string : "", 0);
 	xdebug_append_error_head(&str, html, error_type_str_simple TSRMLS_CC);
-	xdebug_append_error_description(&str, html, error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
+	if (include_decription) {
+		xdebug_append_error_description(&str, html, error_type_str, buffer, error_filename, error_lineno TSRMLS_CC);
+	}
 	xdebug_append_printable_stack(&str, html TSRMLS_CC);
 	xdebug_append_error_footer(&str, html TSRMLS_CC);
 	xdebug_str_add(&str, append_string ? append_string : "", 0);
@@ -513,6 +527,8 @@ void xdebug_init_debugger(TSRMLS_D)
 		XDEBUG_LOG_PRINT(XG(remote_log_file), "E: Could not connect to client. :-(\n");
 	} else if (XG(context).socket == -2) {
 		XDEBUG_LOG_PRINT(XG(remote_log_file), "E: Time-out connecting to client. :-(\n");
+	} else if (XG(context).socket == -3) {
+		XDEBUG_LOG_PRINT(XG(remote_log_file), "E: No permission connecting to client. This could be SELinux related. :-(\n");
 	}
 	if (!XG(remote_enabled)) {
 		xdebug_close_log(TSRMLS_C);
@@ -560,19 +576,12 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 	if (PG(last_error_file)) {
 		free(PG(last_error_file));
 	}
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 2) || PHP_MAJOR_VERSION >= 6
 	PG(last_error_type) = type;
-#endif
 	PG(last_error_message) = strdup(buffer);
 	PG(last_error_file) = strdup(error_filename);
 	PG(last_error_lineno) = error_lineno;
-#if (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3) || PHP_MAJOR_VERSION >= 6
 	error_handling  = EG(error_handling);
 	exception_class = EG(exception_class);
-#else
-	error_handling  = PG(error_handling);
-	exception_class = PG(exception_class);
-#endif
 	/* according to error handling mode, suppress error, throw exception or show it */
 	if (error_handling != EH_NORMAL && EG(in_execution)) {
 		switch (type) {
@@ -601,7 +610,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 		}
 	}
 
-	if (EG(error_reporting) & type) {
+	if ((EG(error_reporting | XG(force_error_reporting))) & type) {
 		/* Log to logger */
 		if (PG(log_errors)) {
 
@@ -614,7 +623,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 		}
 
 		/* Display errors */
-		if (PG(display_errors) && !PG(during_request_startup)) {
+		if ((PG(display_errors) || XG(force_display_errors)) && !PG(during_request_startup)) {
 			char *printable_stack;
 
 			/* We need to see if we have an uncaught exception fatal error now */
@@ -647,7 +656,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 				xdfree(str.d);
 				free(tmp_buf);
 			} else {
-				printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno TSRMLS_CC);
+				printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno, 1 TSRMLS_CC);
 				if (XG(do_collect_errors) && (type != E_ERROR) && (type != E_COMPILE_ERROR) && (type != E_USER_ERROR)) {
 					xdebug_llist_insert_next(XG(collected_errors), XDEBUG_LLIST_TAIL(XG(collected_errors)), printable_stack);
 				} else {
@@ -657,7 +666,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 			}
 		} else if (XG(do_collect_errors)) {
 			char *printable_stack;
-			printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno TSRMLS_CC);
+			printable_stack = get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno, 1 TSRMLS_CC);
 			xdebug_llist_insert_next(XG(collected_errors), XDEBUG_LLIST_TAIL(XG(collected_errors)), printable_stack);
 		}
 	}
@@ -667,45 +676,27 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 
 	/* Check for the pseudo exceptions to allow breakpoints on PHP error statuses */
 	if (XG(remote_enabled) && XG(breakpoints_allowed)) {
-		if (xdebug_hash_find(XG(context).exception_breakpoints, error_type_str, strlen(error_type_str), (void *) &extra_brk_info)) {
+		if (
+			xdebug_hash_find(XG(context).exception_breakpoints, error_type_str, strlen(error_type_str), (void *) &extra_brk_info) ||
+			xdebug_hash_find(XG(context).exception_breakpoints, "*", 1, (void *) &extra_brk_info)
+		) {
 			if (xdebug_handle_hit_value(extra_brk_info)) {
-				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), (char *) error_filename, error_lineno, XDEBUG_BREAK, error_type_str, buffer)) {
+				char *type_str = xdebug_sprintf("%ld", type);
+
+				if (!XG(context).handler->remote_breakpoint(&(XG(context)), XG(stack), (char *) error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer)) {
 					XG(remote_enabled) = 0;
 				}
+
+				xdfree(type_str);
 			}
 		}
 	}
 	xdfree(error_type_str);
 
-#if PHP_VERSION_ID < 50400
-
-	/* Bail out if we can't recover */
-	switch (type) {
-		case E_CORE_ERROR:
-		/* no break - intentionally */
-		case E_ERROR:
-#if PHP_VERSION_ID >= 50200 
-		case E_RECOVERABLE_ERROR:
-#endif
-		/*case E_PARSE: the parser would return 1 (failure), we can bail out nicely */
-		case E_COMPILE_ERROR:
-		case E_USER_ERROR:
-			EG(exit_status) = 255;
-#if HAVE_PHP_MEMORY_USAGE
-			/* restore memory limit */
-# if PHP_VERSION_ID >= 50200 
-			zend_set_memory_limit(PG(memory_limit));
-# else
-			AG(memory_limit) = PG(memory_limit);
-# endif
-#endif
-			zend_objects_store_mark_destructed(&EG(objects_store) TSRMLS_CC);
-			zend_bailout();
-			return;
+	if (type & XG(halt_level) & XDEBUG_ALLOWED_HALT_LEVELS) {
+		type = E_USER_ERROR;
 	}
 
-#else
-
 	/* Bail out if we can't recover */
 	switch (type) {
 		case E_CORE_ERROR:
@@ -742,8 +733,6 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 			break;
 	}
 
-#endif
-
 	if (PG(track_errors) && EG(active_symbol_table)) {
 		zval *tmp;
 
@@ -758,7 +747,7 @@ void xdebug_error_cb(int type, const char *error_filename, const uint error_line
 	efree(buffer);
 }
 
-/* {{{ proto array xdebug_print_function_stack([string message])
+/* {{{ proto array xdebug_print_function_stack([string message [, int options])
    Displays a stack trace */
 PHP_FUNCTION(xdebug_print_function_stack)
 {
@@ -766,16 +755,17 @@ PHP_FUNCTION(xdebug_print_function_stack)
 	int message_len;
 	function_stack_entry *i;
 	char *tmp;
+	long options = 0;
   
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|s", &message, &message_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|sl", &message, &message_len, &options) == FAILURE) {
 		return;
 	}
  
 	i = xdebug_get_stack_frame(0 TSRMLS_CC);
 	if (message) {
-		tmp = get_printable_stack(PG(html_errors), 0, message, i->filename, i->lineno TSRMLS_CC);
+		tmp = get_printable_stack(PG(html_errors), 0, message, i->filename, i->lineno, !(options & XDEBUG_STACK_NO_DESC) TSRMLS_CC);
 	} else {
-		tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno TSRMLS_CC);
+		tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno, !(options & XDEBUG_STACK_NO_DESC) TSRMLS_CC);
 	}
 	php_printf("%s", tmp);
 	xdfree(tmp);
@@ -790,7 +780,7 @@ PHP_FUNCTION(xdebug_get_formatted_function_stack)
 	char *tmp;
 
 	i = xdebug_get_stack_frame(0 TSRMLS_CC);
-	tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno TSRMLS_CC);
+	tmp = get_printable_stack(PG(html_errors), 0, "user triggered", i->filename, i->lineno, 1 TSRMLS_CC);
 	RETVAL_STRING(tmp, 1);
 	xdfree(tmp);
 }
@@ -945,11 +935,7 @@ static void xdebug_build_fname(xdebug_func *tmp, zend_execute_data *edata TSRMLS
 				tmp->function = xdstrdup(edata->function_state.function->common.function_name);
 			}
 		} else {
-#if PHP_VERSION_ID >= 50399
 			switch (edata->opline->extended_value) {
-#else
-			switch (edata->opline->op2.u.constant.value.lval) {
-#endif
 				case ZEND_EVAL:
 					tmp->type = XFUNC_EVAL;
 					break;
@@ -1034,6 +1020,7 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 	/* Call user function locations */
 	if (
 		!tmp->filename &&
+		XG(stack) &&
 		XDEBUG_LLIST_TAIL(XG(stack)) &&
 		XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack))) &&
 		((function_stack_entry*) XDEBUG_LLIST_VALP(XDEBUG_LLIST_TAIL(XG(stack))))->filename
@@ -1044,14 +1031,9 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 	if (!tmp->filename) {
 		tmp->filename = xdstrdup("UNKNOWN?");
 	}
-#if HAVE_PHP_MEMORY_USAGE
 	tmp->prev_memory = XG(prev_memory);
-	tmp->memory = XG_MEMORY_USAGE();
+	tmp->memory = zend_memory_usage(0 TSRMLS_CC);
 	XG(prev_memory) = tmp->memory;
-#else
-	tmp->memory = 0;
-	tmp->prev_memory = 0;
-#endif
 	tmp->time   = xdebug_get_utime();
 	tmp->lineno = 0;
 	tmp->prev   = 0;
@@ -1086,7 +1068,6 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 			 * works for both internal and user defined functions.
 			 * op_array->num_args works only for user defined functions so
 			 * we're not using that here. */
-#if PHP_VERSION_ID >= 50300
 			void **curpos = NULL;
 			if ((!edata->opline) || ((edata->opline->opcode == ZEND_DO_FCALL_BY_NAME) || (edata->opline->opcode == ZEND_DO_FCALL))) {
 				curpos = edata->function_state.arguments;
@@ -1099,13 +1080,6 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 				arguments_wanted = arguments_sent;
 				p = curpos = NULL;
 			}
-#else
-			if (EG(argument_stack).top >= 2) {
-				p = EG(argument_stack).top_element - 2;
-				arguments_sent = (ulong) *p;
-				arguments_wanted = arguments_sent;
-			}
-#endif
 
 			if (tmp->user_defined == XDEBUG_EXTERNAL) {
 				arguments_wanted = op_array->num_args;
@@ -1121,6 +1095,7 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 			for (i = 0; i < arguments_sent; i++) {
 				tmp->var[tmp->varc].name = NULL;
 				tmp->var[tmp->varc].addr = NULL;
+				tmp->var[tmp->varc].is_variadic = 0;
 
 				/* Because it is possible that more parameters are sent, then
 				 * actually wanted  we can only access the name in case there
@@ -1129,22 +1104,18 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 					if (op_array->arg_info[i].name) {
 						tmp->var[tmp->varc].name = xdstrdup(op_array->arg_info[i].name);
 					}
+#if PHP_VERSION_ID >= 50600
+					if (op_array->arg_info[i].is_variadic) {
+						tmp->var[tmp->varc].is_variadic = 1;
+					}
+#endif
 				}
 
 				if (XG(collect_params)) {
-#if PHP_VERSION_ID >= 50300
 					if (p) {
 						param = (zval **) p++;				
 						tmp->var[tmp->varc].addr = *param;
 					}
-#else
-					param = NULL;
-					if (zend_ptr_stack_get_arg(tmp->varc + 1, (void**) &param TSRMLS_CC) == SUCCESS) {
-						if (param) {
-							tmp->var[tmp->varc].addr = *param;
-						}
-					}
-#endif
 				}
 				tmp->varc++;
 			}
@@ -1157,6 +1128,7 @@ function_stack_entry *xdebug_add_stack_frame(zend_execute_data *zdata, zend_op_a
 						tmp->var[tmp->varc].name = xdstrdup(op_array->arg_info[i].name);
 					}
 					tmp->var[tmp->varc].addr = NULL;
+					tmp->var[tmp->varc].is_variadic = 0;
 					tmp->varc++;
 				}
 			}
@@ -1307,20 +1279,37 @@ PHP_FUNCTION(xdebug_get_function_stack)
 		/* Add parameters */
 		MAKE_STD_ZVAL(params);
 		array_init(params);
+		add_assoc_zval_ex(frame, "params", sizeof("params"), params);
+
 		for (j = 0; j < i->varc; j++) {
+			int variadic_opened = 0;
+
+			if (i->var[j].is_variadic) {
+				zval *vparams;
+
+				MAKE_STD_ZVAL(vparams);
+				array_init(vparams);
+
+				if (i->var[j].name) {
+					add_assoc_zval(params, i->var[j].name, vparams);
+				} else {
+					add_index_zval(params, j, vparams);
+				}
+				params = vparams;
+				variadic_opened = 1;
+			}
 			if (i->var[j].addr) {
 				argument = xdebug_get_zval_value(i->var[j].addr, 0, NULL);
 			} else {
-				argument = xdstrdup("");
+				argument = xdstrdup("???");
 			}
-			if (i->var[j].name) {
+			if (i->var[j].name && !variadic_opened) {
 				add_assoc_string_ex(params, i->var[j].name, strlen(i->var[j].name) + 1, argument, 1);
 			} else {
 				add_index_string(params, j, argument, 1);
 			}
 			xdfree(argument);
 		}
-		add_assoc_zval_ex(frame, "params", sizeof("params"), params);
 
 		if (i->include_filename) {
 			add_assoc_string_ex(frame, "include_filename", sizeof("include_filename"), i->include_filename, 1);
