@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -86,10 +86,7 @@ char *xdebug_sprintf(const char* fmt, ...)
 	char   *new_str;
 	int     size = 1;
 	va_list args;
-	char   *orig_locale;
 
-	orig_locale = xdstrdup(setlocale(LC_ALL, NULL));
-	setlocale(LC_ALL, "C");
 	new_str = (char *) xdmalloc(size);
 
 	for (;;) {
@@ -109,8 +106,6 @@ char *xdebug_sprintf(const char* fmt, ...)
 		}
 		new_str = (char *) xdrealloc(new_str, size);
 	}
-	setlocale(LC_ALL, orig_locale);
-	xdfree(orig_locale);
 
 	return new_str;
 }
