@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -47,11 +47,7 @@ static void dump_hash_elem(zval *z, char *name, long index, char *elem, int html
 
 		if (html) {
 			val = xdebug_get_zval_value_fancy(NULL, z, &len, 0, NULL TSRMLS_CC);
-#if HAVE_PHP_MEMORY_USAGE
 			xdebug_str_add(str, xdebug_sprintf("<td colspan='3' bgcolor='#eeeeec'>"), 1);
-#else
-			xdebug_str_add(str, xdebug_sprintf("<td colspan='2' bgcolor='#eeeeec'>"), 1);
-#endif
 			xdebug_str_addl(str, val, len, 0);
 			xdebug_str_add(str, "</td>", 0);
 		} else {
@@ -62,11 +58,7 @@ static void dump_hash_elem(zval *z, char *name, long index, char *elem, int html
 	} else {
 		/* not found */
 		if (html) {
-#if HAVE_PHP_MEMORY_USAGE
 			xdebug_str_add(str, "<td colspan='3' bgcolor='#eeeeec'><i>undefined</i></td>", 0);
-#else
-			xdebug_str_add(str, "<td colspan='2' bgcolor='#eeeeec'><i>undefined</i></td>", 0);
-#endif
 		} else {
 			xdebug_str_add(str, xdebug_sprintf("\n   $%s['%s'] is undefined", name, elem), 1);
 		}
@@ -77,23 +69,16 @@ static void dump_hash_elem(zval *z, char *name, long index, char *elem, int html
 	}
 }
 
-static int dump_hash_elem_va(void *pDest XDEBUG_ZEND_HASH_APPLY_TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+static int dump_hash_elem_va(void *pDest TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
 {
 	int html;
 	char *name;
 	xdebug_str *str;
-#if defined(ZTS) && (!defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300)
-	void ***tsrm_ls;
-#endif
 
 	name = va_arg(args, char *);
 	html = va_arg(args, int);
 	str =  va_arg(args, xdebug_str *);
 
-#if defined(ZTS) && (!defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300)
-	tsrm_ls = va_arg(args, void ***);
-#endif
-
 	if (hash_key->nKeyLength == 0) {
 		dump_hash_elem(*((zval **) pDest), name, hash_key->h, NULL, html, str TSRMLS_CC);
 	} else {
@@ -120,11 +105,7 @@ static void dump_hash(xdebug_llist *l, char *name, int name_len, int html, xdebu
 	}
 
 	if (html) {
-#if HAVE_PHP_MEMORY_USAGE
 		xdebug_str_add(str, xdebug_sprintf("<tr><th colspan='5' align='left' bgcolor='#e9b96e'>Dump <i>$%s</i></th></tr>\n", name), 1);
-#else
-		xdebug_str_add(str, xdebug_sprintf("<tr><th colspan='4' align='left' bgcolor='#e9b96e'>Dump <i>$%s</i></th></tr>\n", name), 1);
-#endif
 	} else {
 		xdebug_str_add(str, xdebug_sprintf("\nDump $%s", name), 1);
 	}
@@ -133,12 +114,7 @@ static void dump_hash(xdebug_llist *l, char *name, int name_len, int html, xdebu
 
 	while (elem != NULL) {
 		if (ht && (*((char *) (elem->ptr)) == '*')) {
-
-#if defined(ZTS) && (!defined(PHP_VERSION_ID) || PHP_VERSION_ID < 50300)
-			zend_hash_apply_with_arguments(ht, dump_hash_elem_va, 4, name, html, str TSRMLS_CC);
-#else
 			zend_hash_apply_with_arguments(ht TSRMLS_CC, dump_hash_elem_va, 3, name, html, str);
-#endif
 		} else if (ht && zend_hash_find(ht, elem->ptr, strlen(elem->ptr) + 1, (void **) &z) == SUCCESS) {
 			dump_hash_elem(*z, name, 0, elem->ptr, html, str TSRMLS_CC);
 		} else if(XG(dump_undefined)) {
