@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2013 Derick Rethans                               |
+   | Copyright (c) 2002-2015 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,18 +12,21 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
    +----------------------------------------------------------------------+
-   | Authors:  Derick Rethans <derick@xdebug.org>                         |
+   | Authors: Derick Rethans <derick@xdebug.org>                          |
    +----------------------------------------------------------------------+
  */
-
 #ifndef XDEBUG_TRACING_H
 #define XDEBUG_TRACING_H
 
-char* xdebug_return_trace_stack_retval(function_stack_entry* i, zval* retval TSRMLS_DC);
+#include "php.h"
+#include "php_xdebug.h"
+
+char* xdebug_return_trace_stack_retval(function_stack_entry* i, int fnr, zval* retval TSRMLS_DC);
 #if PHP_VERSION_ID >= 50500
 char* xdebug_return_trace_stack_generator_retval(function_stack_entry* i, zend_generator* generator TSRMLS_DC);
 #endif
 char* xdebug_return_trace_assignment(function_stack_entry *i, char *varname, zval *retval, char *op, char *file, int fileno TSRMLS_DC);
+FILE *xdebug_trace_open_file(char *fname, long options, char **used_fname TSRMLS_DC);
 
 void xdebug_trace_function_begin(function_stack_entry *fse, int function_nr TSRMLS_DC);
 void xdebug_trace_function_end(function_stack_entry *fse, int function_nr TSRMLS_DC);
